/*
 * OpenI2CRADIO
 * Rotary encoder routine.
 * Copyright (C) 2013-10-21 K.Ohta <whatisthis.sowhat ai gmail.com>
 * License: GPL2+LE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2,
 *  or (at your option) any later version.
 *  This library / program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this library; see the file COPYING. If not, write to the
 *  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  As a special exception, if you link this(includeed from sdcc) library
 *  with other files, some of which are compiled with SDCC,
 *  to produce an executable, this library does not by itself cause
 *  the resulting executable to be covered by the GNU General Public License.
 *  This exception does not however invalidate any other reasons why
 *  the executable file might be covered by the GNU General Public License.
 */
#include "rencoder.h"

unsigned char renc_state_a;
unsigned char renc_state_b;
unsigned char renc_dir;
unsigned char renc_count;
unsigned char renc_reset_count;

void rencoder_up(unsigned char dir)
{
   if(dir == RENC_NONE) return;
   if(dir == renc_dir) {
       renc_count++;
   } else {
       renc_count = 0;
       renc_dir = dir;
   }
}

void rencoder_init(void)
{
   RENC_TRIS_A = 1;
   RENC_TRIS_B = 1;
   renc_count = 0;
   renc_dir = RENC_NONE;
   renc_state_a = 0;
   renc_state_b = 0;
   renc_reset_count = 0;
}


void rencoder_restart(void)
{
}

void rencoder_start(void)
{
   T1CON = 0b10110100;// RD16, T1RUN=0, 1/32 fOSC, TNR1CS=0, TMR1ON=0;
   TMR1H = 63036 >> 8; // Tick = 4us, Count = 2500
   TMR1L = 63036 & 255;
   PIR1bits.TMR1IF = 0;
   PIE1bits.TMR1IE = 1;
   IPR1bits.TMR1IP = 1;
   T1CONbits.TMR1ON = 1;
   rencoder_init();
}

void rencoder_stop(void)
{
   T1CON = 0b10110100;
   TMR1H = 0;
   TMR1L = 0;
   PIR1bits.TMR1IF = 0;
   PIE1bits.TMR1IE = 0;
   IPR1bits.TMR1IP = 1;
   rencoder_init();
}


void rencoder_count(void)
{
   unsigned char dir = 0;
   
   if(RENC_PH_B != 0) {
      if(renc_state_a != 0){
	 if(RENC_PH_A == 0) {
	    dir = RENC_RIGHT;
	 }
      } else {
	 if(RENC_PH_A != 0) {
	    dir = RENC_LEFT;
	 }
      }
   }
   if(dir != RENC_NONE) {
      if(dir == renc_dir) {
	 renc_count++;
      } else {
	 renc_count = 0;
	 renc_dir = dir;
      }
     renc_reset_count = 0;
   } else{
     renc_reset_count++;
     if(renc_reset_count >= 100) { // 1000ms
	renc_count = 0;
	renc_dir = RENC_NONE;
	renc_state_a = 0;
	renc_state_b = 0;
	renc_reset_count = 0;
     }
   }
   
   
//   rencoder_up(dir);
   renc_state_a = RENC_PH_A;
   renc_state_b = RENC_PH_B;
}

	 
	
